*=================================================================================================================================================
* Date : 2024-01-04
* Stata version : Stata 18 SE
*
* This program creates dataset without DIF for a randomized controlled trial scenario
*
* ado-files needed : - simirt (version 4.3 August 29, 2019, available on OSF)
*
  * outputs : scenario_1,scenario_2,scenario_3,scenario_4, for N=100/200/300
*
*
* Warning : To obtain reproduce the data obtained in the .csv files in this repository, use 'simirt_setseed.ado' instead of 'simirt.ado'
*
*
*================================================================================================================================================

* Load simirt.ado
adopath+"/home/corentin/Documents/These/Recherche/Simulations/Modules/"

* Set data output folder path
local path = "/home/corentin/Documents/These/Recherche/Simulations/Data/NoDIF"



* Scenarios with : n = 100
*==========================

local path = "/home/corentin/Documents/These/Recherche/Simulations/Data/NoDIF/N100"
local Nn = 100

** Scenario 1: J = 4 items / M = 2 modalities / N=100 per group / TT=treatment variable

* Scenario 1A : H_0 is TRUE

di "SCENARIO 1A - N=100"

forvalues replication = 1/1000 {
  	if mod(`replication',10)==0 {
  	di "replication = `replication'"
    }
		mat D= (-0.84 \ -0.25 \ 0.25 \ 0.84)
		qui simirt, nbobs(`Nn') mu(0) cov(1) dim(4) pcm(D) clear
		qui gen TT = 0
		tempfile grp0
		qui save `grp0',replace

    mat D= (-0.84 \ -0.25 \ 0.25 \ 0.84)
		qui simirt, nbobs(`Nn') mu(0) cov(1) dim(4) pcm(D) clear
		qui gen TT = 1
		tempfile grp1
		qui save `grp1',replace

		clear
		use `grp0'
		qui append using `grp1'
		drop id
		qui gen id = _n
		order(id)
		qui gen replication = `replication'
		if `replication'==1{
			tempfile data
			qui save `data'
		}
		else{
			qui append using `data'
			qui save `data',replace
		}
	}
	export delimited using "`path'/scenario_1A_100.csv", replace


  * Scenario 1B : H_0 is FALSE / Effect size = 0.2

	di "SCENARIO 1B - N=100"

	forvalues replication = 1/1000 {
	  	if mod(`replication',10)==0 {
	  	di "replication = `replication'"
	    }
  		mat D= (-0.84 \ -0.25 \ 0.25 \ 0.84)
  		qui simirt, nbobs(`Nn') mu(0) cov(1) dim(4) pcm(D) clear
  		qui gen TT = 0
  		tempfile grp0
  		qui save `grp0',replace

      mat D= (-0.84 \ -0.25 \ 0.25 \ 0.84)
  		qui simirt, nbobs(`Nn') mu(0.2) cov(1) dim(4) pcm(D) clear
  		qui gen TT = 1
  		tempfile grp1
  		qui save `grp1',replace

  		clear
  		use `grp0'
  		qui append using `grp1'
  		drop id
  		qui gen id = _n
  		order(id)
  		qui gen replication = `replication'
  		if `replication'==1{
  			tempfile data
  			qui save `data'
  		}
  		else{
  			qui append using `data'
  			qui save `data',replace
  		}
  	}
  	export delimited using "`path'/scenario_1B_100.csv", replace

* Scenario 1C : H_0 is FALSE / Effect size = 0.4

di "SCENARIO 1C - N=100"

forvalues replication = 1/1000 {
  	if mod(`replication',10)==0 {
  	di "replication = `replication'"
    }
  		mat D= (-0.84 \ -0.25 \ 0.25 \ 0.84)
  		qui simirt, nbobs(`Nn') mu(0) cov(1) dim(4) pcm(D) clear
  		qui gen TT = 0
  		tempfile grp0
  		qui save `grp0',replace

      mat D= (-0.84 \ -0.25 \ 0.25 \ 0.84)
  		qui simirt, nbobs(`Nn') mu(0.4) cov(1) dim(4) pcm(D) clear
  		qui gen TT = 1
  		tempfile grp1
  		qui save `grp1',replace

  		clear
  		use `grp0'
  		qui append using `grp1'
  		drop id
  		qui gen id = _n
  		order(id)
  		qui gen replication = `replication'
  		if `replication'==1{
  			tempfile data
  			qui save `data'
  		}
  		else{
  			qui append using `data'
  			qui save `data',replace
  		}
  	}
  	export delimited using "`path'/scenario_1C_100.csv", replace

* Scenario 1D : H_0 is FALSE / Effect size = -0.2

  	di "SCENARIO 1D - N=100"

  	forvalues replication = 1/1000 {
  	  	if mod(`replication',10)==0 {
  	  	di "replication = `replication'"
  	    }
    		mat D= (-0.84 \ -0.25 \ 0.25 \ 0.84)
    		qui simirt, nbobs(`Nn') mu(0) cov(1) dim(4) pcm(D) clear
    		qui gen TT = 0
    		tempfile grp0
    		qui save `grp0',replace

        mat D= (-0.84 \ -0.25 \ 0.25 \ 0.84)
    		qui simirt, nbobs(`Nn') mu(-0.2) cov(1) dim(4) pcm(D) clear
    		qui gen TT = 1
    		tempfile grp1
    		qui save `grp1',replace

    		clear
    		use `grp0'
    		qui append using `grp1'
    		drop id
    		qui gen id = _n
    		order(id)
    		qui gen replication = `replication'
    		if `replication'==1{
    			tempfile data
    			qui save `data'
    		}
    		else{
    			qui append using `data'
    			qui save `data',replace
    		}
    	}
    	export delimited using "`path'/scenario_1D_100.csv", replace

  * Scenario 1E : H_0 is FALSE / Effect size = -0.4

  di "SCENARIO 1E - N=100"

  forvalues replication = 1/1000 {
    	if mod(`replication',10)==0 {
    	di "replication = `replication'"
      }
    		mat D= (-0.84 \ -0.25 \ 0.25 \ 0.84)
    		qui simirt, nbobs(`Nn') mu(0) cov(1) dim(4) pcm(D) clear
    		qui gen TT = 0
    		tempfile grp0
    		qui save `grp0',replace

        mat D= (-0.84 \ -0.25 \ 0.25 \ 0.84)
    		qui simirt, nbobs(`Nn') mu(-0.4) cov(1) dim(4) pcm(D) clear
    		qui gen TT = 1
    		tempfile grp1
    		qui save `grp1',replace

    		clear
    		use `grp0'
    		qui append using `grp1'
    		drop id
    		qui gen id = _n
    		order(id)
    		qui gen replication = `replication'
    		if `replication'==1{
    			tempfile data
    			qui save `data'
    		}
    		else{
    			qui append using `data'
    			qui save `data',replace
    		}
    	}
    	export delimited using "`path'/scenario_1E_100.csv", replace




**------------------------------------------------------------------------------------**


** Scenario 2: J = 4 items / M = 4 modalities / N=100 per group / TT=treatment variable

    * Scenario 2A : H_0 is TRUE
		di "SCENARIO 2A - N=100"

		forvalues replication = 1/1000 {
		  	if mod(`replication',10)==0 {
		  	di "replication = `replication'"
		    }
    		mat D= (-1.84,-0.84,0.16 \ -1.25,-0.25,0.75 \ -0.75,0.25,1.25 \ 0.16,0.84,1.84)
    		qui simirt, nbobs(`Nn') mu(0) cov(1) dim(4) pcm(D) clear
    		qui gen TT = 0
    		tempfile grp0
    		qui save `grp0',replace

        mat D= (-1.84,-0.84,0.16 \ -1.25,-0.25,0.75 \ -0.75,0.25,1.25 \ 0.16,0.84,1.84)
    		qui simirt, nbobs(`Nn') mu(0) cov(1) dim(4) pcm(D) clear
    		qui gen TT = 1
    		tempfile grp1
    		qui save `grp1',replace

    		clear
    		use `grp0'
    		qui append using `grp1'
    		drop id
    		qui gen id = _n
    		order(id)
    		qui gen replication = `replication'
    		if `replication'==1{
    			tempfile data
    			qui save `data'
    		}
    		else{
    			qui append using `data'
    			qui save `data',replace
    		}
    	}
    	export delimited using "`path'/scenario_2A_100.csv", replace


      * Scenario 2B : H_0 is FALSE / Effect size = 0.2
			di "SCENARIO 2B - N=100"

			forvalues replication = 1/1000 {
			  	if mod(`replication',10)==0 {
			  	di "replication = `replication'"
			    }
          mat D= (-1.84,-0.84,0.16 \ -1.25,-0.25,0.75 \ -0.75,0.25,1.25 \ 0.16,0.84,1.84)
      		qui simirt, nbobs(`Nn') mu(0) cov(1) dim(4) pcm(D) clear
      		qui gen TT = 0
      		tempfile grp0
      		qui save `grp0',replace

          mat D= (-1.84,-0.84,0.16 \ -1.25,-0.25,0.75 \ -0.75,0.25,1.25 \ 0.16,0.84,1.84)
      		qui simirt, nbobs(`Nn') mu(0.2) cov(1) dim(4) pcm(D) clear
      		qui gen TT = 1
      		tempfile grp1
      		qui save `grp1',replace

      		clear
      		use `grp0'
      		qui append using `grp1'
      		drop id
      		qui gen id = _n
      		order(id)
      		qui gen replication = `replication'
      		if `replication'==1{
      			tempfile data
      			qui save `data'
      		}
      		else{
      			qui append using `data'
      			qui save `data',replace
      		}
      	}
      	export delimited using "`path'/scenario_2B_100.csv", replace

    * Scenario 2C : H_0 is FALSE / Effect size = 0.4
		di "SCENARIO 2C - N=100"

		forvalues replication = 1/1000 {
		  	if mod(`replication',10)==0 {
		  	di "replication = `replication'"
		    }
          mat D= (-1.84,-0.84,0.16 \ -1.25,-0.25,0.75 \ -0.75,0.25,1.25 \ 0.16,0.84,1.84)
      		qui simirt, nbobs(`Nn') mu(0) cov(1) dim(4) pcm(D) clear
      		qui gen TT = 0
      		tempfile grp0
      		qui save `grp0',replace

          mat D= (-1.84,-0.84,0.16 \ -1.25,-0.25,0.75 \ -0.75,0.25,1.25 \ 0.16,0.84,1.84)
      		qui simirt, nbobs(`Nn') mu(0.4) cov(1) dim(4) pcm(D) clear
      		qui gen TT = 1
      		tempfile grp1
      		qui save `grp1',replace

      		clear
      		use `grp0'
      		qui append using `grp1'
      		drop id
      		qui gen id = _n
      		order(id)
      		qui gen replication = `replication'
      		if `replication'==1{
      			tempfile data
      			qui save `data'
      		}
      		else{
      			qui append using `data'
      			qui save `data',replace
      		}
      	}
      	export delimited using "`path'/scenario_2C_100.csv", replace



* Scenario 2D : H_0 is FALSE / Effect size = -0.2
  			di "SCENARIO 2D - N=100"

  			forvalues replication = 1/1000 {
  			  	if mod(`replication',10)==0 {
  			  	di "replication = `replication'"
  			    }
            mat D= (-1.84,-0.84,0.16 \ -1.25,-0.25,0.75 \ -0.75,0.25,1.25 \ 0.16,0.84,1.84)
        		qui simirt, nbobs(`Nn') mu(0) cov(1) dim(4) pcm(D) clear
        		qui gen TT = 0
        		tempfile grp0
        		qui save `grp0',replace

            mat D= (-1.84,-0.84,0.16 \ -1.25,-0.25,0.75 \ -0.75,0.25,1.25 \ 0.16,0.84,1.84)
        		qui simirt, nbobs(`Nn') mu(-0.2) cov(1) dim(4) pcm(D) clear
        		qui gen TT = 1
        		tempfile grp1
        		qui save `grp1',replace

        		clear
        		use `grp0'
        		qui append using `grp1'
        		drop id
        		qui gen id = _n
        		order(id)
        		qui gen replication = `replication'
        		if `replication'==1{
        			tempfile data
        			qui save `data'
        		}
        		else{
        			qui append using `data'
        			qui save `data',replace
        		}
        	}
        	export delimited using "`path'/scenario_2D_100.csv", replace

* Scenario 2E : H_0 is FALSE / Effect size = -0.4
  		di "SCENARIO 2E - N=100"

  		forvalues replication = 1/1000 {
  		  	if mod(`replication',10)==0 {
  		  	di "replication = `replication'"
  		    }
            mat D= (-1.84,-0.84,0.16 \ -1.25,-0.25,0.75 \ -0.75,0.25,1.25 \ 0.16,0.84,1.84)
        		qui simirt, nbobs(`Nn') mu(0) cov(1) dim(4) pcm(D) clear
        		qui gen TT = 0
        		tempfile grp0
        		qui save `grp0',replace

            mat D= (-1.84,-0.84,0.16 \ -1.25,-0.25,0.75 \ -0.75,0.25,1.25 \ 0.16,0.84,1.84)
        		qui simirt, nbobs(`Nn') mu(-0.4) cov(1) dim(4) pcm(D) clear
        		qui gen TT = 1
        		tempfile grp1
        		qui save `grp1',replace

        		clear
        		use `grp0'
        		qui append using `grp1'
        		drop id
        		qui gen id = _n
        		order(id)
        		qui gen replication = `replication'
        		if `replication'==1{
        			tempfile data
        			qui save `data'
        		}
        		else{
        			qui append using `data'
        			qui save `data',replace
        		}
        	}
        	export delimited using "`path'/scenario_2E_100.csv", replace








**------------------------------------------------------------------------------------**


** Scenario 3: J = 7 items / M = 2 modalities / N=100 per group / TT=treatment variable

* Scenario 3A : H_0 is TRUE
di "SCENARIO 3A - N=100"

forvalues replication = 1/1000 {
  	if mod(`replication',10)==0 {
  	di "replication = `replication'"
    }
		mat D= (-1.15 \ -0.67 \ -0.32 \ 0 \ 0.32 \ 0.67 \ 1.15)
		qui simirt, nbobs(`Nn') mu(0) cov(1) dim(7) pcm(D) clear
		qui gen TT = 0
		tempfile grp0
		qui save `grp0',replace

    mat D= (-1.15 \ -0.67 \ -0.32 \ 0 \ 0.32 \ 0.67 \ 1.15)
		qui simirt, nbobs(`Nn') mu(0) cov(1) dim(7) pcm(D) clear
		qui gen TT = 1
		tempfile grp1
		qui save `grp1',replace

		clear
		use `grp0'
		qui append using `grp1'
		drop id
		qui gen id = _n
		order(id)
		qui gen replication = `replication'
		if `replication'==1{
			tempfile data
			qui save `data'
		}
		else{
			qui append using `data'
			qui save `data',replace
		}
	}
	export delimited using "`path'/scenario_3A_100.csv", replace


  * Scenario 3B : H_0 is FALSE / Effect size = 0.2
	di "SCENARIO 3B - N=100"

	forvalues replication = 1/1000 {
	  	if mod(`replication',10)==0 {
	  	di "replication = `replication'"
	    }
      mat D= (-1.15 \ -0.67 \ -0.32 \ 0 \ 0.32 \ 0.67 \ 1.15)
  		qui simirt, nbobs(`Nn') mu(0) cov(1) dim(7) pcm(D) clear
  		qui gen TT = 0
  		tempfile grp0
  		qui save `grp0',replace

      mat D= (-1.15 \ -0.67 \ -0.32 \ 0 \ 0.32 \ 0.67 \ 1.15)
  		qui simirt, nbobs(`Nn') mu(0.2) cov(1) dim(7) pcm(D) clear
  		qui gen TT = 1
  		tempfile grp1
  		qui save `grp1',replace

  		clear
  		use `grp0'
  		qui append using `grp1'
  		drop id
  		qui gen id = _n
  		order(id)
  		qui gen replication = `replication'
  		if `replication'==1{
  			tempfile data
  			qui save `data'
  		}
  		else{
  			qui append using `data'
  			qui save `data',replace
  		}
  	}
  	export delimited using "`path'/scenario_3B_100.csv", replace

* Scenario 3C : H_0 is FALSE / Effect size = 0.4
di "SCENARIO 3C - N=100"

forvalues replication = 1/1000 {
  	if mod(`replication',10)==0 {
  	di "replication = `replication'"
    }
      mat D= (-1.15 \ -0.67 \ -0.32 \ 0 \ 0.32 \ 0.67 \ 1.15)
  		qui simirt, nbobs(`Nn') mu(0) cov(1) dim(7) pcm(D) clear
  		qui gen TT = 0
  		tempfile grp0
  		qui save `grp0',replace

      mat D= (-1.15 \ -0.67 \ -0.32 \ 0 \ 0.32 \ 0.67 \ 1.15)
  		qui simirt, nbobs(`Nn') mu(0.4) cov(1) dim(7) pcm(D) clear
  		qui gen TT = 1
  		tempfile grp1
  		qui save `grp1',replace

  		clear
  		use `grp0'
  		qui append using `grp1'
  		drop id
  		qui gen id = _n
  		order(id)
  		qui gen replication = `replication'
  		if `replication'==1{
  			tempfile data
  			qui save `data'
  		}
  		else{
  			qui append using `data'
  			qui save `data',replace
  		}
  	}
  	export delimited using "`path'/scenario_3C_100.csv", replace


* Scenario 3D : H_0 is FALSE / Effect size = -0.2
  	di "SCENARIO 3D - N=100"

  	forvalues replication = 1/1000 {
  	  	if mod(`replication',10)==0 {
  	  	di "replication = `replication'"
  	    }
        mat D= (-1.15 \ -0.67 \ -0.32 \ 0 \ 0.32 \ 0.67 \ 1.15)
    		qui simirt, nbobs(`Nn') mu(0) cov(1) dim(7) pcm(D) clear
    		qui gen TT = 0
    		tempfile grp0
    		qui save `grp0',replace

        mat D= (-1.15 \ -0.67 \ -0.32 \ 0 \ 0.32 \ 0.67 \ 1.15)
    		qui simirt, nbobs(`Nn') mu(-0.2) cov(1) dim(7) pcm(D) clear
    		qui gen TT = 1
    		tempfile grp1
    		qui save `grp1',replace

    		clear
    		use `grp0'
    		qui append using `grp1'
    		drop id
    		qui gen id = _n
    		order(id)
    		qui gen replication = `replication'
    		if `replication'==1{
    			tempfile data
    			qui save `data'
    		}
    		else{
    			qui append using `data'
    			qui save `data',replace
    		}
    	}
    	export delimited using "`path'/scenario_3D_100.csv", replace

* Scenario 3E : H_0 is FALSE / Effect size = -0.4
  di "SCENARIO 3E - N=100"

  forvalues replication = 1/1000 {
    	if mod(`replication',10)==0 {
    	di "replication = `replication'"
      }
        mat D= (-1.15 \ -0.67 \ -0.32 \ 0 \ 0.32 \ 0.67 \ 1.15)
    		qui simirt, nbobs(`Nn') mu(0) cov(1) dim(7) pcm(D) clear
    		qui gen TT = 0
    		tempfile grp0
    		qui save `grp0',replace

        mat D= (-1.15 \ -0.67 \ -0.32 \ 0 \ 0.32 \ 0.67 \ 1.15)
    		qui simirt, nbobs(`Nn') mu(-0.4) cov(1) dim(7) pcm(D) clear
    		qui gen TT = 1
    		tempfile grp1
    		qui save `grp1',replace

    		clear
    		use `grp0'
    		qui append using `grp1'
    		drop id
    		qui gen id = _n
    		order(id)
    		qui gen replication = `replication'
    		if `replication'==1{
    			tempfile data
    			qui save `data'
    		}
    		else{
    			qui append using `data'
    			qui save `data',replace
    		}
    	}
    	export delimited using "`path'/scenario_3E_100.csv", replace





**------------------------------------------------------------------------------------**


** Scenario 4: J = 7 items / M = 4 modalities / N=100 per group / TT=treatment variable

    * Scenario 4A : H_0 is TRUE
		di "SCENARIO 4A - N=100"

		forvalues replication = 1/1000 {
		  	if mod(`replication',10)==0 {
		  	di "replication = `replication'"
		    }
    		mat D= (-2.15,-1.15,-0.15 \ -1.67,-0.67,0.33 \ -1.32,-0.32,0.68 \ -1,0,1 \ -0.68,0.32,1.32 \ -0.33,0.67,1.67 \ 0.15,1.15,2.15)
    		qui simirt, nbobs(`Nn') mu(0) cov(1) dim(7) pcm(D) clear
    		qui gen TT = 0
    		tempfile grp0
    		qui save `grp0',replace

        mat D= (-2.15,-1.15,-0.15 \ -1.67,-0.67,0.33 \ -1.32,-0.32,0.68 \ -1,0,1 \ -0.68,0.32,1.32 \ -0.33,0.67,1.67 \ 0.15,1.15,2.15)
    		qui simirt, nbobs(`Nn') mu(0) cov(1) dim(7) pcm(D) clear
    		qui gen TT = 1
    		tempfile grp1
    		qui save `grp1',replace

    		clear
    		use `grp0'
    		qui append using `grp1'
    		drop id
    		qui gen id = _n
    		order(id)
    		qui gen replication = `replication'
    		if `replication'==1{
    			tempfile data
    			qui save `data'
    		}
    		else{
    			qui append using `data'
    			qui save `data',replace
    		}
    	}
    	export delimited using "`path'/scenario_4A_100.csv", replace


    * Scenario 4B : H_0 is FALSE / Effect size = 0.2
		di "SCENARIO 4B - N=100"

		forvalues replication = 1/1000 {
		  	if mod(`replication',10)==0 {
		  	di "replication = `replication'"
		    }
          mat D= (-2.15,-1.15,-0.15 \ -1.67,-0.67,0.33 \ -1.32,-0.32,0.68 \ -1,0,1 \ -0.68,0.32,1.32 \ -0.33,0.67,1.67 \ 0.15,1.15,2.15)
      		qui simirt, nbobs(`Nn') mu(0) cov(1) dim(7) pcm(D) clear
      		qui gen TT = 0
      		tempfile grp0
      		qui save `grp0',replace

          mat D= (-2.15,-1.15,-0.15 \ -1.67,-0.67,0.33 \ -1.32,-0.32,0.68 \ -1,0,1 \ -0.68,0.32,1.32 \ -0.33,0.67,1.67 \ 0.15,1.15,2.15)
      		qui simirt, nbobs(`Nn') mu(0.2) cov(1) dim(7) pcm(D) clear
      		qui gen TT = 1
      		tempfile grp1
      		qui save `grp1',replace

      		clear
      		use `grp0'
      		qui append using `grp1'
      		drop id
      		qui gen id = _n
      		order(id)
      		qui gen replication = `replication'
      		if `replication'==1{
      			tempfile data
      			qui save `data'
      		}
      		else{
      			qui append using `data'
      			qui save `data',replace
      		}
      	}
      	export delimited using "`path'/scenario_4B_100.csv", replace

    * Scenario 4C : H_0 is FALSE / Effect size = 0.4
		di "SCENARIO 4C - N=100"

		forvalues replication = 1/1000 {
		  	if mod(`replication',10)==0 {
		  	di "replication = `replication'"
		    }
          mat D= (-2.15,-1.15,-0.15 \ -1.67,-0.67,0.33 \ -1.32,-0.32,0.68 \ -1,0,1 \ -0.68,0.32,1.32 \ -0.33,0.67,1.67 \ 0.15,1.15,2.15)
      		qui simirt, nbobs(`Nn') mu(0) cov(1) dim(7) pcm(D) clear
      		qui gen TT = 0
      		tempfile grp0
      		qui save `grp0',replace

          mat D= (-2.15,-1.15,-0.15 \ -1.67,-0.67,0.33 \ -1.32,-0.32,0.68 \ -1,0,1 \ -0.68,0.32,1.32 \ -0.33,0.67,1.67 \ 0.15,1.15,2.15)
      		qui simirt, nbobs(`Nn') mu(0.4) cov(1) dim(7) pcm(D) clear
      		qui gen TT = 1
      		tempfile grp1
      		qui save `grp1',replace

      		clear
      		use `grp0'
      		qui append using `grp1'
      		drop id
      		qui gen id = _n
      		order(id)
      		qui gen replication = `replication'
      		if `replication'==1{
      			tempfile data
      			qui save `data'
      		}
      		else{
      			qui append using `data'
      			qui save `data',replace
      		}
      	}
      	export delimited using "`path'/scenario_4C_100.csv", replace


* Scenario 4D : H_0 is FALSE / Effect size = -0.2
    		di "SCENARIO 4D - N=100"

    		forvalues replication = 1/1000 {
    		  	if mod(`replication',10)==0 {
    		  	di "replication = `replication'"
    		    }
              mat D= (-2.15,-1.15,-0.15 \ -1.67,-0.67,0.33 \ -1.32,-0.32,0.68 \ -1,0,1 \ -0.68,0.32,1.32 \ -0.33,0.67,1.67 \ 0.15,1.15,2.15)
          		qui simirt, nbobs(`Nn') mu(0) cov(1) dim(7) pcm(D) clear
          		qui gen TT = 0
          		tempfile grp0
          		qui save `grp0',replace

              mat D= (-2.15,-1.15,-0.15 \ -1.67,-0.67,0.33 \ -1.32,-0.32,0.68 \ -1,0,1 \ -0.68,0.32,1.32 \ -0.33,0.67,1.67 \ 0.15,1.15,2.15)
          		qui simirt, nbobs(`Nn') mu(-0.2) cov(1) dim(7) pcm(D) clear
          		qui gen TT = 1
          		tempfile grp1
          		qui save `grp1',replace

          		clear
          		use `grp0'
          		qui append using `grp1'
          		drop id
          		qui gen id = _n
          		order(id)
          		qui gen replication = `replication'
          		if `replication'==1{
          			tempfile data
          			qui save `data'
          		}
          		else{
          			qui append using `data'
          			qui save `data',replace
          		}
          	}
          	export delimited using "`path'/scenario_4D_100.csv", replace

* Scenario 4E : H_0 is FALSE / Effect size = -0.4
    		di "SCENARIO 4E - N=100"

    		forvalues replication = 1/1000 {
    		  	if mod(`replication',10)==0 {
    		  	di "replication = `replication'"
    		    }
              mat D= (-2.15,-1.15,-0.15 \ -1.67,-0.67,0.33 \ -1.32,-0.32,0.68 \ -1,0,1 \ -0.68,0.32,1.32 \ -0.33,0.67,1.67 \ 0.15,1.15,2.15)
          		qui simirt, nbobs(`Nn') mu(0) cov(1) dim(7) pcm(D) clear
          		qui gen TT = 0
          		tempfile grp0
          		qui save `grp0',replace

              mat D= (-2.15,-1.15,-0.15 \ -1.67,-0.67,0.33 \ -1.32,-0.32,0.68 \ -1,0,1 \ -0.68,0.32,1.32 \ -0.33,0.67,1.67 \ 0.15,1.15,2.15)
          		qui simirt, nbobs(`Nn') mu(-0.4) cov(1) dim(7) pcm(D) clear
          		qui gen TT = 1
          		tempfile grp1
          		qui save `grp1',replace

          		clear
          		use `grp0'
          		qui append using `grp1'
          		drop id
          		qui gen id = _n
          		order(id)
          		qui gen replication = `replication'
          		if `replication'==1{
          			tempfile data
          			qui save `data'
          		}
          		else{
          			qui append using `data'
          			qui save `data',replace
          		}
          	}
          	export delimited using "`path'/scenario_4E_100.csv", replace
